/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.controllers.dict


import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.models.entities.dict.DictSQueryRequest
import com.persagy.server.datacenter.models.entities.dictnew.DefClass
import com.persagy.server.datacenter.models.entities.dictnew.DefFuncId
import com.persagy.server.datacenter.models.entities.dictnew.DefFuncIdGather
import com.persagy.server.datacenter.models.entities.dictnew.DefFuncIdProject
import com.persagy.server.datacenter.models.jms.SynchronousMessage
import com.persagy.server.datacenter.models.requests.CategoryRequest
import com.persagy.server.service.dict.DictDefClassService
import com.persagy.service.SObjectService
import com.persagy.service.models.requests.SCountRequest
import com.persagy.service.models.requests.SCreateRequest
import com.persagy.service.models.responses.SCountResponse
import com.persagy.service.models.responses.SCreateResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 * 项目化字典
 *
 * @author 张维新
 */
@Tag(name = "项目化字典")
@RestController
@RequestMapping("/dict")
class DictCustomController {

    /** 缓存对象数据服务 - 数据中心到数据中台  */
    var synchronousMessageObjService = SObjectService(SMybatisDao(SynchronousMessage::class.java))

    /**
     * 查询 M 和 L 类型的信息点
     */
    @Operation(summary = "字典-查询所有字典信息 M 和 L 类型,包括自定义信息点", description = "project (项目),building (建筑),floor (楼层),equipment (设备),system(系统),component (部件),space (空间)")
    @PostMapping(value = ["/morl/query"])
    fun queryMOrL(@RequestBody dictTypeList: ArrayList<String>): SQueryResponse<DefFuncId> {
        return DictDefClassService.queryMOrL(dictTypeList)
    }

    /**
     * 获取不能同步到物理世界对象的数量
     */
    @Operation(summary = "获取不能同步到物理世界对象的数量", description = "")
    @PostMapping(value = ["/data-count"])
    fun objectCounts(@RequestBody request: SCountRequest): SCountResponse {
        val messageList =
            synchronousMessageObjService.select(SFilter.eq("projectId", Opt.projectId!!),SFilter.not(SFilter.eq("sign", 2))).order("createTime").exec()
        return SCountResponse(messageList.size.toLong())
    }  // Function objectCounts()

    /**
     * 字典查询所有设备类型、部件所有类型、系统类型、专业类型
     *
     * @param  categoryRequest        查询信息条件
     * @return 查询结果
     */
    @Operation(summary = "类型-字典查询所有设备类型、部件所有类型、系统类型、专业类型", description = "输入设备类型：equipment，输入部件类型：component,输入系统类型：system,输入专业类型：major," +
            "专业和系统：majorSystem，业务空间类型 space ,专业下的设备类型 majorEquipment ,设备和部件equipmentAndComponent,设备类型要以及对应的部件equipmentComponent，专业下的系统，系统下的设备majorSystemEquip ,资产 property ")
    @PostMapping(value = ["/category"])
    fun categoryQuery(@RequestBody categoryRequest: CategoryRequest): SQueryResponse<DefClass> {
        return DictDefClassService.equipQuery(categoryRequest)
    }

    /**
     * 项目字典
     *
     * @param request   查询条件
     * @return 查询结构
     */
    @Operation(summary ="查询项目下类型的信息点",description = " 租户（VOTn）")
    @PostMapping(value = ["/query"])
    fun categoryInfoQuery(@RequestBody request: DictSQueryRequest): SQueryResponse<DefFuncIdGather>{
        return DictDefClassService.categoryInfoQuery(request)
    }

    /**
     * 项目定制化设置
     *
     * @param request   信息点对象列表
     * @return 返回结果
     */
    @Operation(summary = "设置-项目定制化字典", description = "")
    @PostMapping(value = ["/setting-dic-project"])
    fun settingDict(@RequestBody request: SCreateRequest<DefFuncIdGather>): SCreateResponse<DefFuncIdGather> {
        return DictDefClassService.settingDict(request)
    }

    /**
     * 需采集信息同步到所有项目
     */
    @Operation(summary = "需采集信息同步到所有项目", description = "")
    @PostMapping(value = ["/syn-dic-project"])
    fun synDict(): SCreateResponse<DefFuncIdGather> {
        return DictDefClassService.synDictProject()
    }
}